"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDependenciesFromPackage = exports.resolveDependencies = exports.generateXml = void 0;
/* eslint-disable no-restricted-syntax,newline-per-chained-call */
const xmlbuilder2_1 = require("xmlbuilder2");
const fs = __importStar(require("fs"));
const os = __importStar(require("os"));
const path_1 = __importDefault(require("path"));
const child_process_1 = require("child_process");
const shared_1 = require("@cubejs-backend/shared");
const MINIMAL_VERSION = '3.6.3';
const RECOMMENDED_VERSION = '3.8.0';
function generateXml(dependencies) {
    const root = (0, xmlbuilder2_1.create)()
        .ele('project', {
        xmlns: 'http://maven.apache.org/POM/4.0.0',
        'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
        'xsi:schemaLocation': 'http://maven.apache.org/POM/4.0.0 http://maven.apache.org/xsd/maven-4.0.0.xsd',
    })
        // project->
        .ele('modelVersion').txt('4.0.0').up()
        .ele('groupId').txt('com.mycompany.app').up()
        .ele('artifactId').txt('my-app').up()
        .ele('version').txt('1.0-SNAPSHOT').up()
        // // project->properties
        // .ele('properties')
        // .ele('maven.compiler.source').txt('1.8').up()
        // .ele('maven.compiler.target').txt('1.8').up()
        // .up()
        // project->dependencies
        .ele('dependencies');
    for (const dependency of dependencies) {
        // project->dependencies->dependency
        const depRoot = root.ele('dependency');
        for (const [key, value] of Object.entries(dependency)) {
            depRoot.ele(key).txt(value);
        }
        depRoot.up();
    }
    return root.up().end({ prettyPrint: true });
}
exports.generateXml = generateXml;
async function getSystemMavenVersion() {
    const mvnOutput = (0, child_process_1.spawnSync)('mvn', ['--version'], {
        stdio: 'pipe',
        encoding: 'utf8'
    });
    if (mvnOutput.status === 0) {
        const result = mvnOutput.stdout.match(/Apache Maven (\d+).(\d+).(\d+)/);
        if (result) {
            return {
                binary: 'mvn',
                version: [
                    parseInt(result[1], 10),
                    parseInt(result[2], 10),
                    parseInt(result[3], 10),
                ]
            };
        }
    }
    return null;
}
async function getExternalMaven() {
    const cwd = path_1.default.join(process.cwd(), 'download');
    const binary = path_1.default.join(cwd, `apache-maven-${RECOMMENDED_VERSION}`, 'bin', 'mvn');
    if (fs.existsSync(binary)) {
        return {
            binary,
            version: [RECOMMENDED_VERSION.split('.')]
        };
    }
    await (0, shared_1.downloadAndExtractFile)(`https://apache-mirror.rbc.ru/pub/apache/maven/maven-3/3.8.0/binaries/apache-maven-${RECOMMENDED_VERSION}-bin.tar.gz`, {
        cwd,
        showProgress: true,
    });
    return {
        binary,
        version: [RECOMMENDED_VERSION.split('.')]
    };
}
async function getMaven(options) {
    const systemBinary = await getSystemMavenVersion();
    if (systemBinary) {
        const [major, minor, patch] = systemBinary.version;
        const [minMajor, minMinor, minPatch] = MINIMAL_VERSION.split('.').map((v) => parseInt(v, 10));
        if (major > minMajor) {
            return systemBinary;
        }
        if (major === minMajor) {
            if (minor >= minMinor) {
                return systemBinary;
            }
            else if (minor === minMinor && patch >= minPatch) {
                return systemBinary;
            }
        }
    }
    if (options.showOutput) {
        if (systemBinary) {
            console.log(`[maven-resolve] Unable to use mvn from the system, current: ${systemBinary.version.join('.')}, ` +
                `minimal version: ${MINIMAL_VERSION}`);
        }
        else {
            console.log(`[maven-resolve] Unable to find mvn from the system, trying to download ${RECOMMENDED_VERSION}`);
        }
    }
    return getExternalMaven();
}
async function installDependencies(pathToPom, options) {
    const { binary } = await getMaven(options);
    // https://search.maven.org/artifact/de.qaware.maven/go-offline-maven-plugin/1.2.8/jar
    const mvnOutput = (0, child_process_1.spawnSync)(binary, ['de.qaware.maven:go-offline-maven-plugin:1.2.8:resolve-dependencies', '-f', pathToPom], {
        stdio: options.showOutput ? 'inherit' : 'pipe',
        encoding: 'utf8'
    });
    if (mvnOutput.status === 0) {
        return;
    }
    throw new Error(`Unable to resolve maven dependencies, mvn exited with: "${mvnOutput.status}" code`);
}
async function resolveDependencies(dependecies, options) {
    const pathToPom = path_1.default.resolve(os.tmpdir(), 'cubejs-pom.xml');
    fs.writeFileSync(pathToPom, generateXml(dependecies));
    await installDependencies(pathToPom, options);
}
exports.resolveDependencies = resolveDependencies;
function getDependenciesFromPackage() {
    const pathToLock = path_1.default.join(process.cwd(), 'package.json');
    if (!fs.existsSync(pathToLock)) {
        throw new Error(`Unable to find package.json file in current working directory: ${process.cwd()}`);
    }
    const content = fs.readFileSync(pathToLock, {
        encoding: 'utf-8'
    });
    const pkg = JSON.parse(content);
    if (pkg) {
        if (pkg.java && pkg.java.dependencies) {
            return pkg.java.dependencies;
        }
    }
    return [];
}
exports.getDependenciesFromPackage = getDependenciesFromPackage;
//# sourceMappingURL=maven.js.map